//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------
#include "m8c.h"
#include "psocapi.h"
#include "stdlib.h"
#include <arith.h>

#define debug	0//1

#pragma interrupt_handler PSoC_GPIO_ISR
#pragma interrupt_handler Counter32_1_ISR

#define Fclk 47972265	//6494      // 23988247	//24000000
#define _FTOA_TOO_LARGE -2 /* |input| > 2147483520 */

#define LeftButton  0xA0	//0xEF 	//P2_4  	11101111
#define CentButton  0x30	//0x7F 	//P2_7  	01111111
#define RightButton 0x90	//0xDF 	//P2_5  	11011111
#define LC			0x20
#define CR			0x10
#define LR			0x80
#define LCR			0x00
#define ButtMask	0xB0	//					10110000
#define EEAddr 		0
#define Cal			8
#define Button_ 	(PRT2DR & ButtMask); DelaymS(200)
#define Lo			1
#define Hi			0	
#define Dual		2
#define CompareRatio 3/4

char CButton;
char LCDContr;
char Mode=Lo;
char Key=0xFF;
char Div;
char Dcod[]= {0,1,3,7};
char Dmul[]= {80,40,20,10};
//char Dstr[]= "8421";

void DelaymS(int Delay);
void TestFreq(void);
void Display(void);
void Save(void);
void Config_Hi(void);	// Connect Counter32_1 Clock to RI0(0) -> Pin_2_0
void Config_Lo(void);	// Connect Counter32_1 Clock to RI0(2) -> Pin_2_2
void SwitchConfig(void);
void ChangeMode(void);
void Setup(void);
void SetHiRef(void);
void SetLoRef(void);
void SetLCD(void);
void Cls00(void);
void Cls10(void);

unsigned long Count2;
unsigned long Period;
unsigned long PeriodT;
unsigned long GateWindow;
unsigned long FxT;

float Fx;
int i;
char buff[16];
char Ready=0;
char Overflow=0;
char VrefHi;
char VrefLo;
char CurrConfig;
char Stereo=0;

void main(void)
{
    LCD_Start();
	E2PROM_E2Read( EEAddr, &LCDContr, 1 );
	E2PROM_E2Read( EEAddr+1, &VrefHi, 1 );
	E2PROM_E2Read( EEAddr+2, &VrefLo, 1 );

	DAC_LCD_Start(DAC_LCD_HIGHPOWER);
	DAC_HiF_Start(DAC_HiF_HIGHPOWER);
	DAC_LoF_Start(DAC_LoF_HIGHPOWER);
	
	if (LCDContr > 23){
	VrefHi=45;
	VrefLo=2;
	LCDContr=18;
	}
	
	DAC_LCD_WriteBlind(LCDContr);
	DAC_HiF_WriteBlind(VrefHi);
	DAC_LoF_WriteBlind(VrefLo);
	
//	Config_Hi();
	Config_Lo();
	
	Div=3;
	PRT3DR=Dcod[Div];	// Divider = 10
	
	Period=200;
	PeriodT=Period;
	
	Counter32_1_EnableInt();
	Counter32_1_WritePeriod(Period);
	Counter32_1_WriteCompareValue(Period/2);
	Counter32_1_Start();
	Counter32_2_Start();
	
	LCD_PrCString("FineFreqMet03.11");
	DelaymS(1000);
	
	INT_MSK0 |= 0x20;
	M8C_EnableGInt;	
	
	for(;;)
	{
		Overflow=0;
	
		while(Ready!=1)
			{
			DelaymS(100);
			Overflow++;
			if(Overflow>10)
				{
				TestFreq();
				Ready=1;
				Overflow=0;
				}
			}		
		Display();
		
			if(Stereo)
			{
			Counter32_1_DisableInt();
			SwitchConfig();
			}
			else Counter32_1_EnableInt();
		
		Ready =0;
	}
}
void Counter32_1_ISR(void)
{		
	Counter32_2_Stop();
	Counter32_2_ReadCounter(&Count2);
 	Counter32_2_WritePeriod(0xffffffff);	
	Counter32_2_Start();
		
	Count2=0xffffffff-Count2-1;//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! -1
//	Count2=0xffffffff-Count2;	//-1;
	
	Fx = (float)Fclk/Count2;
	Fx = (float)Period*Fx/2;

	fp2ulong(Fx, &PeriodT);	
	
		
	if (PeriodT<100)
		{
		Period=PeriodT/2;
		Period=Period*2;
		}
	else
		{
		Period=PeriodT/32;
		Period=Period*32;
		}

	//Period=Period*2;	//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!11

	Counter32_1_WritePeriod(Period);
	Counter32_1_WriteCompareValue(Period/2);

	Ready=1;
}

void DelaymS(int Delay)
{
int i;
for(i=0;i<Delay;i++) LCD_Delay50uTimes(20);
}
void TestFreq(void)
	{	
	LCD_Position(CurrConfig,15);
	LCD_PrCString("T");
	Counter32_1_Stop();
	Counter32_1_WritePeriod(0xffffffff);
	Counter32_1_Start();
	DelaymS(991);
	Counter32_1_Stop();
	Counter32_1_ReadCounter(&Count2);
	
	Count2=0xffffffff-Count2;		
	
	if (Count2<100)
		{
		Period=Count2/2;
		Period=Period*2;
		}
		else
		{
		Period=Count2/32;
		Period=Period*32;
		}
	
	Fx = Count2;
	
//	Period=Period*2;	//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Counter32_1_WritePeriod(Period);
	Counter32_1_WriteCompareValue(Period/2);
	Counter32_1_Start();
	}
void Display(void)
{	
	#if(debug)
			LCD_Position(CurrConfig^1,0);
			LCD_PrCString("P");
			ultoa(buff, Period,10);
			LCD_PrString(buff);

			LCD_PrCString(" T");
			ultoa(buff, PeriodT,10);
			LCD_PrString(buff);
			LCD_PrCString("        ");
	#endif					
			LCD_Position(CurrConfig,0);	
			
			switch(Mode)
				{
					case Lo:
						LCD_PrString(ftoa(Fx, &i));
						break;
					case Hi:
						fp2ulong(Fx, &FxT);
						ultoa(buff,FxT*Dmul[Div], 10);
						LCD_PrString(buff);
						LCD_PrCString("/");
						itoa(buff, Dmul[Div],10);
						LCD_PrString(buff);
						break;
					case Dual:
						fp2ulong(Fx, &FxT);
						if(CurrConfig==Hi)
							{
							ultoa(buff,FxT*Dmul[Div], 10);
							LCD_PrString(buff);
							LCD_PrCString("/");
							itoa(buff, Dmul[Div],10);
							LCD_PrString(buff);						
							}
							else
							{
							ultoa(buff,FxT, 10);
							LCD_PrString(buff);
							}
							
						break;
				}											
			LCD_PrCString(" Hz             ");
			
			
			
			
}			
	
void Save(void)
{
char bError; 

bError=E2PROM_bE2Write(EEAddr, &LCDContr, 1, 25);
bError=E2PROM_bE2Write(EEAddr+1, &VrefHi, 1, 25);
bError=E2PROM_bE2Write(EEAddr+2, &VrefLo, 1, 25);
LCD_Position(1,13);
LCD_PrHexByte(bError);
}

void PSoC_GPIO_ISR(void)
	{
	Key = Button_;
	
	switch(Key)
		{
		case LeftButton:
		Setup();
		break;
			
		case CentButton:
		Div++;
		if(Div >3) Div=0;
		PRT3DR = Dcod[Div];
		break;
			
		case RightButton:	
		ChangeMode();				
		}	
	}

void Config_Hi(void)	// Connect Counter32_1 Clock to RI0(0) -> Pin_2_0
	{
	Counter32_1_Stop();	
	DBB00IN = 0x1C;
	DBB01IN = 0x3C;
	DCB02IN = 0x3C;	
	DCB03IN = 0x3C;		//?? Doesn't changed by IDE allways =0x3C
	Counter32_1_Start();
	CurrConfig = Hi;
	}

void Config_Lo(void)	// Connect Counter32_1 Clock to RI0(2) -> Pin_2_2
	{
	Counter32_1_Stop();
	DBB00IN = 0x1E;
	DBB01IN = 0x3E;
	DCB02IN	= 0x3E;	
	DCB03IN = 0x3E;			//?? Doesn't changed by IDE allways =0x3C	
	Counter32_1_Start();
	CurrConfig = Lo;
	}
	
void SwitchConfig(void)
{
if (CurrConfig==Hi)Config_Lo(); else Config_Hi();
}

void ChangeMode(void)
{	
			Mode++;
			if (Mode>2)Mode=0;			

			Cls00();
			if(Mode == Hi)
			{
			Config_Hi();
			Stereo=0;
			LCD_Position(1,0);
			LCD_PrCString("High Frequency  ");
			}
			else if(Mode == Lo)
			{			
			Config_Lo();
			Stereo=0;
			LCD_Position(0,0);
			LCD_PrCString("Low  Frequency  ");
			}
			else if(Mode == Dual)
			{			
			Stereo=1;
			LCD_PrCString("       Dual     ");
			}
	DelaymS(1000);		
						
}

void Setup(void)
{
			Cls00();

			LCD_PrCString(" RHi  RLo  LCD  ");
			Key=ButtMask;
			while(Key==ButtMask)
			{
				Key = Button_;
				
				if (Key==LeftButton) 
				{
				SetHiRef();
				}
				else if (Key==CentButton) 
				{
				SetLoRef();
				}
				else if (Key==RightButton)
				{
				SetLCD();
				}
			}
			Save();
			DelaymS(1000);
			Cls00();
			
}

void SetHiRef(void)
{
			LCD_Position(0,0);
						//"                "
			LCD_PrCString("        Vref Hi ");
			
			LCD_Position(1,0);

			LCD_PrCString("   <    >  Save ");

			DelaymS(500);
			Key=0xFF;
			while(Key!=RightButton)
				{
				Key = Button_;
				if(Key==LeftButton)VrefHi--;
				if(VrefHi>63)VrefHi = 0;
				if(Key==CentButton)VrefHi++;
				if(VrefHi>62)VrefHi=62;
				
				DAC_HiF_WriteBlind(VrefHi);
				itoa(buff, VrefHi*Cal, 10);
				
				LCD_Position(0,0);
				LCD_PrString(buff);
				LCD_PrCString("0mV ");
				}
	//		Save();
}

void SetLoRef(void)
{
			LCD_Position(0,0);
						//"                "
			LCD_PrCString("        Vref Lo ");
			
			LCD_Position(1,0);

			LCD_PrCString("   <    >  Save ");
			
			DelaymS(500);
			Key=0xFF;
			while(Key!=RightButton)
				{
				Key = Button_;		
				if(Key==LeftButton)VrefLo--;
				if(VrefLo>63)VrefLo = 0;
				if(Key==CentButton)VrefLo++;
				if(VrefLo>62)VrefLo=62;
				
				DAC_LoF_WriteBlind(VrefLo);
				itoa(buff, VrefLo*Cal, 10);
				
				LCD_Position(0,0);
				LCD_PrString(buff);	
				LCD_PrCString("0mV ");
				}	
	//		Save();
}
void SetLCD(void)
{
			LCD_Position(0,0);
						//"                "
			LCD_PrCString("    LCD Contrast");
			
			LCD_Position(1,0);

			LCD_PrCString("   <    >  Save ");
			
			DelaymS(500);
			Key=0xFF;
			while(Key!=RightButton)
				{
				Key = Button_;	
				if(Key==LeftButton)LCDContr--;
				if(LCDContr>23)LCDContr = 0;
				if(Key==CentButton)LCDContr++;
				if(LCDContr>22)LCDContr=22;
				
				DAC_LCD_WriteBlind(LCDContr);
				itoa(buff, LCDContr, 10);
				
				LCD_Position(0,0);
				LCD_PrString(buff);	
				LCD_PrCString(" ");
				}	
	//		Save();
}
void Cls00(void)
{
	LCD_Position(1,0);
	LCD_PrCString("                ");
	LCD_Position(0,0);
	LCD_PrCString("                ");
	LCD_Position(0,0);
}


