;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME: DAC_LCD.asm
;;   Version: 4.3, Updated on 2010/9/19 at 23:40:5
;;  Generated by PSoC Designer 5.1.1875.0
;;
;;  DESCRIPTION: DAC6 User Module software implementation file.
;;
;;  NOTE: User Module APIs conform to the fastcall16 convention for marshalling
;;        arguments and observe the associated "Registers are volatile" policy.
;;        This means it is the caller's responsibility to preserve any values
;;        in the X and A registers that are still needed after the API functions
;;        returns. For Large Memory Model devices it is also the caller's 
;;        responsibility to perserve any value in the CUR_PP, IDX_PP, MVR_PP and 
;;        MVW_PP registers. Even though some of these registers may not be modified
;;        now, there is no guarantee that will remain the case in future releases.
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress Semiconductor 2010. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

include "DAC_LCD.inc"
include "memory.inc"
include "m8c.inc"

;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------

export  DAC_LCD_Start
export _DAC_LCD_Start
export  DAC_LCD_SetPower
export _DAC_LCD_SetPower

export  DAC_LCD_WriteBlind
export _DAC_LCD_WriteBlind
export  DAC_LCD_WriteStall
export _DAC_LCD_WriteStall

export  DAC_LCD_Stop
export _DAC_LCD_Stop

;; -----------------------------------------------------------------
;;                         Register Definitions
;; -----------------------------------------------------------------
;;
;; Uses 1 Switched Cap Block configured as shown. This API depends
;; on knowing the exact personalization of CR0 and CR3 bitfields
;; for time efficiency.
;;
;; * For a Mask/Val pair, this simply indicates that the value is
;;   determined by the user either through config-time parameteriza-
;;   tion or run-time manipulation.
;;
;; BIT FIELD         Mask/Val Function
;; -----------------    ----- --------------------
;; CR0.FCap             80/1  Feedback cap size 32
;; CR0.ClockPhase       40/0  Normal phase
;; CR0.ASign            20/*  User parameter
;; CR0.ACap             1F/*  User parameter
;;
;; CR1.ACMux            E0/2  (SCA) A:VRef High, C:Don't Care
;; CR1.AMux             E0/4  (SCB) VRef High
;; CR1.BCap             1F/0  Prune B-input branch
;;
;; CR2.AnalogBus        80/*  User Parameter: Output Bus Enable
;; CR2.CmpBus           40/0  Comparator Bus Disable
;; CR2.AutoZero         20/1  Auto-Zero enabled on Phi 1
;; CR2.CCap             1F/0  Prune C-input branch
;;
;; CR3.ARefSelect       C0/0  Use AGND (to invert)
;; CR3.FSW1             20/1  Feedback Cap Used
;; CR3.FSW2             10/1  Feedback Cap Grounded for AZ
;; CR3.BMux             0C/0  (SCA) Don't Care - this branch pruned
;; CR3.BSW              08/0  (SCB) Don't Care - this branch pruned
;; CR3.BMux             04/0  (SCB) Don't Care - this branch pruned
;; CR3.PWR              03/*  User Parameter: Power, def=OFF
;;

;-----------------------------------------------
;  Constant Definitions
;-----------------------------------------------
cOFFSET:   equ 31               ; Conversion term for offset binary to 2's C
bPWRMASK:  equ 03h              ; Power bitfield in Switched Cap CR3 reg
bCR3:      equ 30h              ; Except for power bits, CR3 ALWAYS looks
                                ;    like this regardless of SC block type
                                ;    or where the DAC gets mapped.


AREA UserModules (ROM, REL)

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: DAC_LCD_Start
;  FUNCTION NAME: DAC_LCD_SetPower
;
;  DESCRIPTION:
;    Start / SetPower - Applies power setting to the module's SC blocks
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;    A contains the power setting 0=Off, 1=Low, 2=Med, 3=High
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;
 DAC_LCD_Start:
_DAC_LCD_Start:
 DAC_LCD_SetPower:
_DAC_LCD_SetPower:
        RAM_PROLOGUE RAM_USE_CLASS_1 
        and A, bPWRMASK
        or  A, bCR3             ; Set all other bits in addition to power
        mov reg[DAC_LCD_CR3], A
        RAM_EPILOGUE RAM_USE_CLASS_1 
        ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: DAC_LCD_WriteBlind
;
;  DESCRIPTION:
;    Modify the DAC's update value without worrying about the clocks
;    Lowest overhead, but output may not settle to correct value until the
;    phi2 of next full cycle following the write.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;    The accumulator, A, contains the input in the appropriate format.
;    The data format is determined by the setting of the DataFormat parameter
;    in the Device Editor.
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;
 DAC_LCD_WriteBlind:
_DAC_LCD_WriteBlind:

  RAM_PROLOGUE RAM_USE_CLASS_1 
  IF DAC_LCD_OFFSETBINARY
    ;; Data is an unsigned byte value in [0..62] (i.e., 63 unique values).
    ;; Following converts it to 2's complement:
    sub  A, cOFFSET         ; Apply the offset
  ENDIF
  IF DAC_LCD_OFFSETBINARY | DAC_LCD_TWOSCOMPLEMENT
    ;; Data is a byte in standard 2's complement form with value in [-31..+31]
    ;; Following converts it to Sign & Magnitude form "00smmmmm"
    ;;   where sign, "s", is 1 for negative numbers; 0 for positive
    ;;   and "m" is the magnitude.
    asl  A                  ; Multiply by 2 and put sign in Carry flag
    jnc  BlindPositive
    ;; Neg to pos by "Invert & Add 1" procedure, but data is shifted!
    cpl  A                  ; bit 0 is a "1" so, following 1 byte "inc" works
    inc  A                  ;   (otherwise, we'd have to "add A, 2")
    or   A, 40h             ; Make it negative by forcing sign bit
    jmp  BlindMagSet
BlindPositive:
    nop
    nop
    nop
    jmp  BlindMagSet
BlindMagSet:
    asr  A                  ; Divide by two to finish up
  ENDIF

    ;; Data is in Sign & Magnitude form.
    ;; Set FCap and ClockPhase bits
    or   A, DAC_LCD_CR0_HIBITS
    mov  reg[DAC_LCD_CR0], A
    RAM_EPILOGUE RAM_USE_CLASS_1 
    ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: DAC_LCD_WriteStall
;
;  DESCRIPTION:
;    Modify the DAC's update value, stalling the CPU if necessary.
;    This routine should be used with fast analog clocks or when the
;    resulting interrupt latencies, comparable to the update period,
;    can be tolerated comfortably.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;    The accumulator, A, contains the input in the appropriate format.
;    The data format is determined by the setting of the DataFormat parameter
;    in the Device Editor.
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.    
;

 DAC_LCD_WriteStall:
_DAC_LCD_WriteStall:

  RAM_PROLOGUE RAM_USE_CLASS_1 
  IF DAC_LCD_OFFSETBINARY
    ;; Data is an unsigned byte value in [0..62] (i.e., 63 unique values).
    ;; Following converts it to 2's complement:
    sub  A, cOFFSET         ; Apply the offset
  ENDIF
  IF DAC_LCD_OFFSETBINARY | DAC_LCD_TWOSCOMPLEMENT
    ;; Data is a byte in standard 2's complement form with value in [-31..+31]
    ;; Following converts it to Sign & Magnitude form "00smmmmm"
    ;;   where sign, "s", is 1 for negative numbers; 0 for positive
    ;;   and "m" is the magnitude.
    asl  A                  ; Multiply by 2 and put sign in Carry flag
    jnc  StallPositive
    cpl  A                  ; "Invert" step of complement 2's complement
    inc  A                  ; "Add 1"  step of complement 2's complement
    or   A, 40h             ; Make it negative
    jmp  StallMagSet
StallPositive:
    nop
    nop
    nop
    jmp  StallMagSet
StallMagSet:
    asr  A                  ; Divide by two to finish conversion
  ENDIF

    ;; Data is in Sign & Magnitude form.
    ;; Set FCap and ClockPhase bits
    or   A, DAC_LCD_CR0_HIBITS
    M8C_Stall
    mov  reg[DAC_LCD_CR0], A
    M8C_Unstall
    RAM_EPILOGUE RAM_USE_CLASS_1 
    ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: DAC_LCD_Stop
;
;  DESCRIPTION:
;    Turns off power to the SC block
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: none
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;

 DAC_LCD_Stop:
_DAC_LCD_Stop:
    RAM_PROLOGUE RAM_USE_CLASS_1 
    and reg[DAC_LCD_CR3], ~bPWRMASK
    RAM_EPILOGUE RAM_USE_CLASS_1 
    ret
.ENDSECTION

; End of File DAC_LCD.asm
