//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------
#include "m8c.h"
#include "psocapi.h"
#include "stdlib.h"
#include <arith.h>

#define debug	0//1

#pragma interrupt_handler PSoC_GPIO_ISR
#pragma interrupt_handler Counter32_1_ISR

//#define Fclk 47972265	//6494      // 23988247	//24000000
#define Fclk 48000000
#define _FTOA_TOO_LARGE -2 /* |input| > 2147483520 */

#define LeftButton  0xA0	//0xEF 	//P2_4  	11101111
#define CentButton  0x30	//0x7F 	//P2_7  	01111111
#define RightButton 0x90	//0xDF 	//P2_5  	11011111
#define LC			0x20
#define CR			0x10
#define LR			0x80
#define LCR			0x00
#define ButtMask	0xB0	//					10110000
#define EEAddr 		0
#define Cal			80
#define Button_ 	(PRT2DR & ButtMask); DelaymS(200)
#define DefaultCal	0.9994222

#define Hi			0
#define Lo			1
#define Dual		2
#define CalXtal		3

char Mode=Lo;
char Key=0xFF;
char Div;
char Ready=0;
char Overflow=0;
char LCDContr;
char VrefHi;
char VrefLo;
char CurrConfig;
char Dcod[]= {0,1,3,7};
char Dmul[]= {80,40,20,10};
char buff[16];
int i;

unsigned long Count2;
unsigned long GateWindow;
float Ccal=DefaultCal;
float Fx;
float Fs=0;

void DelaymS(int Delay);
void TestFreq(void);
void Display(void);
void Save(void);
void Config_Hi(void);	// Connect Counter32_1 Clock to RI0(0) -> Pin_2_0
void Config_Lo(void);	// Connect Counter32_1 Clock to RI0(2) -> Pin_2_2
void SwitchConfig(void);
void ChangeMode(void);
void Setup(void);
void SetHiRef(void);
void SetLoRef(void);
void SetLCD(void);
void Cls00(void);
void Cls10(void);
void Calibrate(void);
void SetMode(void);

void main(void)
{
    LCD_Start();
	
	E2PROM_E2Read( EEAddr,   &LCDContr, 1 ); //E2PROM fail to write if RAM usage >90 bytes
	E2PROM_E2Read( EEAddr+1, &VrefHi, 1 );
	E2PROM_E2Read( EEAddr+2, &VrefLo, 1 );
	E2PROM_E2Read( EEAddr+3, &Mode, 1 );
	E2PROM_E2Read( EEAddr+4, &Div, 1 );
	E2PROM_E2Read( EEAddr+5, (char *)&Ccal, 4 );
	
	DAC_LCD_Start(DAC_LCD_HIGHPOWER);
	DAC_HiF_Start(DAC_HiF_HIGHPOWER);
	DAC_LoF_Start(DAC_LoF_HIGHPOWER);
			
	if (LCDContr > 23)
	{
	LCDContr=0;
//	LCDContr=18;	
	VrefHi=45;
	VrefLo=6;
	Mode = Lo;	
	Div  = 3;
	Ccal=DefaultCal;
	}
	
	DAC_LCD_WriteBlind(LCDContr);
	DAC_HiF_WriteBlind(VrefHi);
	DAC_LoF_WriteBlind(VrefLo);
	
	Div=3;
	PRT3DR=Dcod[Div];	// Divider = 10
	
	GateWindow=4;
	
	Counter32_1_EnableInt();
	Counter32_1_WritePeriod(GateWindow+GateWindow/4);
	Counter32_1_WriteCompareValue(GateWindow/4);
	Counter32_1_Start();
	Counter32_2_Start();
	
	LCD_PrCString("FineMeter 05.11");
	LCD_Position(1,0);

	LCD_PrCString("Cal ");
	LCD_PrString(ftoa(Ccal, &i));
	DelaymS(1000);
	
	SetMode();
	
	INT_MSK0 |= 0x20;
	M8C_EnableGInt;	
	
	for(;;)
	{
		Overflow=0;
	
		while(Ready!=1)
			{
			DelaymS(100);
			Overflow++;
			if(Overflow>20)
				{
				TestFreq();
				Ready=1;
				}
			}		
		Display();
		
		
			
		Ready =0;
		if(Mode==CalXtal)Calibrate();
	}
}

void Counter32_1_ISR(void)
{		
	Counter32_2_Stop();
	Counter32_2_ReadCounter(&Count2);
 	Counter32_2_WritePeriod(0xffffffff);	
	Counter32_2_Start();
		
	Count2=0xffffffff-Count2;	//-1;	
	Fx = (float)Fclk/Count2;
	
	Fx = (float)GateWindow*Fx;
	
	Fx=Fx*Ccal;
	
	fp2ulong(Fx, &GateWindow);	
		
	GateWindow=GateWindow & 0xfffffff0;

	Counter32_1_WritePeriod(GateWindow+GateWindow/8);
	Counter32_1_WriteCompareValue(GateWindow/8);
	
	Ready =1;
}

void DelaymS(int Delay)
{
int i;
for(i=0;i<Delay;i++) LCD_Delay50uTimes(20);
}
void TestFreq(void)
	{	
	LCD_Position(CurrConfig,15);
	LCD_PrCString("T");

	Counter32_1_Stop();
	Counter32_2_Stop();
	Counter32_1_WritePeriod(0xffffffff);
	Counter32_1_Start();
	DelaymS(125);
	Counter32_1_Stop();
	Counter32_1_ReadCounter(&GateWindow);

	GateWindow=0xffffffff-GateWindow;
	Fx = GateWindow*8;
	
	GateWindow = GateWindow*4;

	Counter32_2_WritePeriod(0xffffffff);
	
	Counter32_1_WritePeriod(GateWindow+GateWindow/8);
	Counter32_1_WriteCompareValue(GateWindow/8);
	
	Counter32_1_Start();
	Counter32_2_Start();
	}
void Display(void)
{	
	int i;	
	unsigned long FxT;
	#if(debug)
			LCD_Position(CurrConfig^1, 0);
			LCD_PrCString("G ");

			ultoa(buff, GateWindow, 10);
			LCD_PrString(buff);
			LCD_PrCString("             ");
	#endif					
			LCD_Position(CurrConfig,0);	
			
			switch(Mode)
				{
					case Lo:
						LCD_PrString(ftoa(Fx, &i));
						LCD_PrCString("        ");
						break;
						
					case Hi:
						fp2ulong(Fx*Dmul[Div], &FxT);

						ultoa(buff, FxT, 10);
						LCD_PrString(buff);
						
						LCD_PrCString("        ");
						
						LCD_Position(CurrConfig,10);
						LCD_PrCString("/");
						
						itoa(buff, Dmul[Div],10);
						LCD_PrString(buff);
					break;						
				}	
			LCD_Position(CurrConfig,14);	
			LCD_PrCString("Hz");											
}			
	
void Save(void)
{
char bError=0; 

bError+=E2PROM_bE2Write(EEAddr,   &LCDContr, 1, 25);
bError+=E2PROM_bE2Write(EEAddr+1, &VrefHi, 1, 25);
bError+=E2PROM_bE2Write(EEAddr+2, &VrefLo, 1, 25);
bError+=E2PROM_bE2Write(EEAddr+3, &Mode, 1, 25);
bError+=E2PROM_bE2Write(EEAddr+4, &Div, 1, 25);
bError+=E2PROM_bE2Write(EEAddr+5, (char *)&Ccal, 4, 25);

LCD_Position(1,13);
LCD_PrHexByte(bError);
}

void PSoC_GPIO_ISR(void)
	{
	Key = Button_;
	
	switch(Key)
		{
		case LeftButton:
		Setup();
		break;
			
		case CentButton:
		if(Mode==Hi)
		{
		Div--;
		if(Div >3) Div=3;
		PRT3DR = Dcod[Div];
		Save();
		}
		else Mode = CalXtal;
		break;
			
		case RightButton:	
		ChangeMode();
		break;		
		}	
	}

void Config_Hi(void)	// Connect Counter32_1 Clock to RI0(0) -> Pin_2_0
	{
	Counter32_1_Stop();	
	DBB00IN = 0x1C;
	DBB01IN = 0x3C;
	DCB02IN = 0x3C;	
	DCB03IN = 0x3C;		//?? Doesn't changed by IDE allways =0x3C
	Counter32_1_Start();
	CurrConfig = Hi;
	}

void Config_Lo(void)	// Connect Counter32_1 Clock to RI0(2) -> Pin_2_2
	{
	Counter32_1_Stop();
	DBB00IN = 0x1E;
	DBB01IN = 0x3E;
	DCB02IN	= 0x3E;	
	DCB03IN = 0x3E;			//?? Doesn't changed by IDE allways =0x3C	
	Counter32_1_Start();
	CurrConfig = Lo;
	}
	
void SwitchConfig(void)
{
if (CurrConfig==Hi)Config_Lo(); else Config_Hi();
}

void ChangeMode(void)
{	
			Mode++;
			if (Mode>1)Mode=0;			
			SetMode();
}

void SetMode(void)
{
	Cls00();
	LCD_Position(CurrConfig,0);
	switch(Mode)
	{
		case Hi:			
		Config_Hi();
		LCD_PrCString("High Frequency  ");
		break;
		
		case Lo:			
		Config_Lo();
		LCD_PrCString("Low  Frequency  ");
	}	
	Save();
}


void Setup(void)
{
			Cls00();

			LCD_PrCString(" RHi  RLo  LCD  ");
			Key=ButtMask;
			while(Key==ButtMask)
			{
				Key = Button_;
				
				if (Key==LeftButton) 
				{
				SetHiRef();
				}
				else if (Key==CentButton) 
				{
				SetLoRef();
				}
				else if (Key==RightButton)
				{
				SetLCD();
				}
			}
			Save();
			DelaymS(1000);
			Cls00();
			
}

void SetHiRef(void)
{
			LCD_Position(0,0);
						//"                "
			LCD_PrCString("        Vref Hi ");
			
			LCD_Position(1,0);

			LCD_PrCString("   <    >  Save ");

			DelaymS(500);
			Key=0xFF;
			while(Key!=RightButton)
				{
				Key = Button_;
				if(Key==LeftButton)VrefHi--;
				if(VrefHi>63)VrefHi = 0;
				if(Key==CentButton)VrefHi++;
				if(VrefHi>62)VrefHi=62;
				
				DAC_HiF_WriteBlind(VrefHi);
				
				itoa(buff, VrefHi*Cal, 10);
				
				LCD_Position(0,0);
				LCD_PrString(buff);
				LCD_PrCString("mV ");
				}
}

void SetLoRef(void)
{
			LCD_Position(0,0);
						//"                "
			LCD_PrCString("        Vref Lo ");
			
			LCD_Position(1,0);

			LCD_PrCString("   <    >  Save ");
			
			DelaymS(500);
			Key=0xFF;
			while(Key!=RightButton)
				{
				Key = Button_;		
				if(Key==LeftButton)VrefLo--;
				if(VrefLo>63)VrefLo = 0;
				if(Key==CentButton)VrefLo++;
				if(VrefLo>62)VrefLo=62;
				
				DAC_LoF_WriteBlind(VrefLo);

				itoa(buff, VrefLo*Cal, 10);
				
				LCD_Position(0,0);
				LCD_PrString(buff);	
				LCD_PrCString("mV ");
				}	
}
void SetLCD(void)
{
			LCD_Position(0,0);
						//"                "
			LCD_PrCString("    LCD Contrast");
			
			LCD_Position(1,0);

			LCD_PrCString("   <    >  Save ");
			
			DelaymS(500);
			Key=0xFF;
			while(Key!=RightButton)
				{
				Key = Button_;	
				if(Key==LeftButton)LCDContr--;
				if(LCDContr>23)LCDContr = 0;
				if(Key==CentButton)LCDContr++;
				if(LCDContr>22)LCDContr=22;
				
				DAC_LCD_WriteBlind(LCDContr);

				itoa(buff, LCDContr, 10);
				
				LCD_Position(0,0);
				LCD_PrString(buff);	
				LCD_PrCString(" ");
				}	
}
void Cls00(void)
{
	LCD_Position(1,0);
	LCD_PrCString("                ");
	LCD_Position(0,0);
	LCD_PrCString("                ");
	LCD_Position(0,0);
}

void Cls10(void)
{
	LCD_Position(0,0);
	LCD_PrCString("                ");
	LCD_Position(1,0);
	LCD_PrCString("                ");
	LCD_Position(1,0);
}

void Calibrate(void)
{
	
	char n;
	int i;
	float Fs=0;
	Ccal=1.0;
	Key=ButtMask;
	Cls00();
	LCD_PrCString("Calibration...  ");
	
	for(n=0;n<10;n++)
	{
		Overflow=0;
	
		while(Ready!=1)
			{
			DelaymS(100);
			Overflow++;
			if(Overflow>20)
				{
				TestFreq();
				n=0;
				Fs=0;
				Ready=1;
				}
			}		
		Fs+=Fx;
		Ready =0;
	}
	
	Fs=Fs/10;
		
	if		((Fs > 24)    & (Fs < 26)) 
	{
	Ccal=(float)25/Fs;
	LCD_Position(1,0);
	LCD_PrCString("Done @ 25 Hz   ");
	Save();
	}
	else if ((Fs > 49)    & (Fs<51))    
	{
	Ccal=(float)50/Fs;
	LCD_Position(1,0);
	LCD_PrCString("Done @ 50 Hz   ");	
	Save();
	}	
	else if ((Fs > 15990) & (Fs<16010)) 
	{
	Ccal=(float)16000/Fs;
	LCD_Position(1,0);
	LCD_PrCString("Done @ 1600 Hz ");
	Save();
	}
	else 
	{
	LCD_Position(1,0);
	LCD_PrCString(" Fail");	
	}
	
	DelaymS(1000);
		
	Mode=Lo;
	Config_Lo();
	Cls10();
	Save();
}


